package com.sunwin.metro.flatmap;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.bean.MetroDevice;
import com.sunwin.metro.source.mysql.MysqlData;
import com.sunwin.metro.utils.DateUtils;
import com.sunwin.metro.utils.Md5Utils;
import com.sunwin.metro.utils.PerStringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.util.Collector;
import org.apache.hadoop.yarn.webapp.hamlet.Hamlet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.Date;

/**
 * @program: sunwin_metro
 * @description: 把设备属性拆分成一条一条数据
 * @author: xuYJ
 * @create: 2021-01-15 09:10
 **/
public class MetroDevicePropertyFlatmap implements FlatMapFunction<String, MetroDevice> {
    public static final Logger logger = LoggerFactory.getLogger(MetroDevicePropertyFlatmap.class);

    @Override
    public void flatMap(String input, Collector<MetroDevice> collector) throws Exception {
        JSONObject device = JSON.parseObject(input);
        JSONArray dpeList = device.getJSONArray("dpes");
        String deviceTime = DateUtils.stringDateFormat(device.getString("stime"), "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'", "yyyyMMddHHmmssSSS");
        String compName = device.getString("compname");
        String system = "非常规系统";
        String deviceName = "设备名为空";
        if (compName.contains("_")) {
            String[] systems = compName.split("_");
            if (!PerStringUtils.isNullOrEmpty(systems[0])) {
                system = systems[0];
                deviceName = compName.replace(systems[0] + "_", "").trim();
            } else {
                logger.error("异常设备号" + compName);
            }
        } else {
            system = compName;
            deviceName = compName;
        }
        for (Object s : dpeList) {
            JSONObject dpeObject = JSONObject.parseObject(s.toString());
            String collectTime = device.getString("collectTime");
            String compDes = device.getString("compDes");
            String stationName = device.getString("stationName");
            String stationId = device.getString("stationId");
            String subSys = device.getString("subSys");
            String result = dpeObject.getString("val");
            String valType = String.valueOf(dpeObject.getInteger("valType"));
            String dpe = dpeObject.getString("dpe");
            String des = dpeObject.getString("des");
            String startTime = DateUtils.stringDateFormat(dpeObject.getString("stime"), "yyyy.MM.dd HH:mm:ss.SSS", "yyyyMMddHHmmssSSS");
            String user = dpeObject.getString("user");
            String warn = dpeObject.getString("hdl_Abbr");
            String offLineText = dpeObject.getString("offLineText");
            String id = String.valueOf(Md5Utils.getDeviceId(startTime, dpe));
            String attributeType = "非常规属性状态";
            String attributeVal = "系统携带默认属性";
            if (dpe.contains("DI")) {
                attributeType = "DI";
            } else if (dpe.contains("AI")) {
                attributeType = "AI";
            } else if (dpe.contains("DO")) {
                attributeType = "DO";
            } else if (dpe.contains("AO")) {
                attributeType = "AO";
            }
            if (!"非常规属性状态".equals(attributeType)) {
                String[] split = dpe.split(attributeType);
                if (split.length>1 && !PerStringUtils.isNullOrEmpty(split[1].replace(".", " ").trim())) {
                    attributeVal = split[1].replace(".", "").trim();
                }
            }
            collector.collect(new MetroDevice(collectTime, compDes, stationName, deviceTime, compName, id, stationId, result, des, offLineText, valType, dpe, startTime, user, warn, system, deviceName, subSys, attributeType, attributeVal));
        }
    }

    public static void main(String[] args) {
        String a="sw404:FAS_Fiber_LW1.AI";
        String[] ais = a.split("AI");
        System.out.println("Arrays.toString(ais) = " + Arrays.toString(ais));
        System.out.println("ais = " + ais.length);
        System.out.println("ais = " + ais[1]);
    }
}
