package com.sunwin.metro.map;

import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.utils.websocket.Client;
import org.apache.flink.api.common.functions.RichMapFunction;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.configuration.Configuration;

/**
 * @program: sunwin_metro
 * @description: 采集webSocket服务端的设备信息，发送到kafka
 * @author: xuYJ
 * @create: 2021-02-19 11:07
 **/
public class DeviceClientMap extends RichMapFunction<Tuple2<String, String>, String> {
    private transient MapState<String, Integer> mapState;
    @Override
    public void open(Configuration parameters) throws Exception {
        MapStateDescriptor<String, Integer> descriptor = new MapStateDescriptor<>("mapState", TypeInformation.of(String.class), TypeInformation.of(Integer.class));
        mapState = getRuntimeContext().getMapState(descriptor);
        System.out.println("当前分区" + getRuntimeContext().getIndexOfThisSubtask());
        Client.createConnect();
        super.open(parameters);
    }

    @Override
    public String map(Tuple2<String, String> input) throws Exception {
        String inputNew = input.f1.trim();
        if (mapState.isEmpty()) {
            System.out.println("当前分区" + getRuntimeContext().getIndexOfThisSubtask());
            JSONObject object = new JSONObject();
            System.out.println("input = " + input);
            object.put("dpes", inputNew);
            Client.send(object.toJSONString());
            mapState.put(inputNew, 1);
            return "设备号请求成功 - " + inputNew;
        } else if (null != mapState.get(inputNew)) {
            System.out.println("当前分区" + getRuntimeContext().getIndexOfThisSubtask());
            System.out.println("请求失败，重复设备 - " + inputNew);
            return "设备号请求失败，重复设备采集 - " + inputNew;
        } else {
            System.out.println("当前分区" + getRuntimeContext().getIndexOfThisSubtask());
            JSONObject object = new JSONObject();
            object.put("dpes", inputNew);
            Client.send(object.toJSONString());
            mapState.put(inputNew, 1);
            return "设备号请求成功 - " + inputNew;
        }
    }
}
