package com.sunwin.metro.map;

import com.sunwin.metro.bean.MetroDevice;
import com.sunwin.metro.utils.PerStringUtils;
import org.apache.flink.api.common.functions.RichMapFunction;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.metrics.Gauge;
import org.apache.flink.metrics.MetricGroup;

/**
 * @program: sunwin_metro  （该类监控ups系统信息）
 * @description: 自定义设备metic信息，发送监控信息到Grafana可视化展示
 * @author: xuYJ
 * @create: 2021-01-22 10:50
 **/
public class DeviceMetricsMap extends RichMapFunction<MetroDevice, MetroDevice> {
    private transient int valueToExpose = 0;
    private transient String valueToExpose2 ;
    private transient int valueToExpose3 = 0;
    private transient Double valueToExpose4 = 0.0;
    private transient int valueToExpose5 = 0;
    private transient Double valueToExpose6 = 0.0;
    private transient int valueToExpose7 = 0;
    private transient int valueToExpose8 = 0;
    private transient int valueToExpose9 = 0;
    private transient Double valueToExpose10 = 0.0;
    private transient int valueToExpose11 = 0;
    private transient Double valueToExpose12 = 0.0;

    @Override
    public void open(Configuration parameters) throws Exception {
        MetricGroup metricGroup = getRuntimeContext()
                .getMetricGroup();
        metricGroup
                .gauge("sw404:UPS_BAS.AI.BV", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.BVP", new Gauge<String>() {
                    @Override
                    public String getValue() {
                        return valueToExpose2;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.IFV", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose3;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.IFre", new Gauge<Double>() {
                    @Override
                    public Double getValue() {
                        return valueToExpose4;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.IV", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose5;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.OP", new Gauge<Double>() {
                    @Override
                    public Double getValue() {
                        return valueToExpose6;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.OV", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose7;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.RBV", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose8;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.ROC", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose9;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.ROFre", new Gauge<Double>() {
                    @Override
                    public Double getValue() {
                        return valueToExpose10;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.ROV", new Gauge<Integer>() {
                    @Override
                    public Integer getValue() {
                        return valueToExpose11;
                    }
                });
        metricGroup
                .gauge("sw404:UPS_BAS.AI.Temp", new Gauge<Double>() {
                    @Override
                    public Double getValue() {
                        return valueToExpose12;
                    }
                });
        super.open(parameters);
    }

    @Override
    public void close() throws Exception {
        super.close();
    }

    @Override
    public MetroDevice map(MetroDevice metroDevice) throws Exception {
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.BV".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.BVP".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose2 = metroDevice.getAttribute_result();
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.IFV".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose3 = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.IFre".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose4 = Double.parseDouble(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.IV".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose5 = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.OP".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose6 =  Double.parseDouble(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.OV".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose7 = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.RBV".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose8 = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.ROC".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose9 = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.ROFre".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose10 = Double.parseDouble(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.ROV".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose11 = Integer.parseInt(metroDevice.getAttribute_result());
            }
        }
        if (!PerStringUtils.isNullOrEmpty(metroDevice.getOriginal_dpe()) && "sw404:UPS_BAS.AI.Temp".equals(metroDevice.getOriginal_dpe())) {
            if (!PerStringUtils.isNullOrEmpty(metroDevice.getAttribute_result())) {
                valueToExpose12 = Double.parseDouble(metroDevice.getAttribute_result());
            }
        }
        return metroDevice;
    }
}
