package com.sunwin.metro.process;

import com.sunwin.metro.bean.MetroDevice;
import com.sunwin.metro.utils.FileUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

/**
 * @program: sunwin_metro
 * @description: 设备某一属性进行报警过程中输出该设备所有属性
 * @author: xuYJ
 * @create: 2021-03-08 15:09
 **/
public class DeviceAlarmProcess extends KeyedProcessFunction<String, MetroDevice, Tuple2<String, MetroDevice>> {
    private transient ValueState<Integer> valueState;
    private transient MapState<String, Long> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        MapStateDescriptor<String, Long> descriptor = new MapStateDescriptor<>("mapState", TypeInformation.of(String.class), TypeInformation.of(Long.class));
        mapState = getRuntimeContext().getMapState(descriptor);
        ValueStateDescriptor<Integer> descriptor2 = new ValueStateDescriptor<>("valueState", TypeInformation.of(Integer.class));
        valueState = getRuntimeContext().getState(descriptor2);
        super.open(parameters);
    }

    @Override
    public void processElement(MetroDevice metroDevice, Context context, Collector<Tuple2<String, MetroDevice>> collector) throws Exception {
        Integer value = valueState.value();
        String[] words = {"动作", "故障", "屏蔽", "监管", "报警", "离线", "通讯故障", "火警"};
        if (mapState.isEmpty()) {
            if (metroDevice.getDevice_warn().contains("报警")) {
                if (FileUtils.containsWords(metroDevice.getAction_text(), words)) {
                    valueState.update(null == value ? 1 : value + 1);
                    mapState.put(metroDevice.getOriginal_dpe(), Long.valueOf(metroDevice.getAttribute_time()));
                    collector.collect(new Tuple2<>(valueState.value().toString(), metroDevice));
                }
            }
        } else {
            if (FileUtils.containsWords(metroDevice.getAction_text(), words)) {
                if (metroDevice.getDevice_warn().contains("报警")) {
                    mapState.put(metroDevice.getOriginal_dpe(), Long.valueOf(metroDevice.getAttribute_time()));
                }
                collector.collect(new Tuple2<>(valueState.value().toString(), metroDevice));
            } else {
                if (mapState.contains(metroDevice.getOriginal_dpe())) {
                    collector.collect(new Tuple2<>(valueState.value().toString(), metroDevice));
                    mapState.remove(metroDevice.getOriginal_dpe());
                } else {
                    collector.collect(new Tuple2<>(valueState.value().toString(), metroDevice));
                }
            }
        }
    }
}
