package com.sunwin.metro.process;

import com.sunwin.metro.bean.MetroDevice;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

/**
 * @program: sunwin_metro
 * @description: 对设备属性进行去重
 * @author: xuYJ
 * @create: 2021-03-08 14:56
 **/
public class DeviceUnrepeated extends KeyedProcessFunction<String, MetroDevice, MetroDevice> {
    private transient MapState<String, String> mapState;

    @Override
    public void open(Configuration parameters) throws Exception {
        MapStateDescriptor<String, String> descriptor = new MapStateDescriptor<>("mapState", TypeInformation.of(String.class), TypeInformation.of(String.class));
        mapState = getRuntimeContext().getMapState(descriptor);
        super.open(parameters);
    }

    @Override
    public void processElement(MetroDevice metroDevice, Context context, Collector<MetroDevice> collector) throws Exception {
        String metro = metroDevice.getAction_text() + metroDevice.getDevice_warn() + metroDevice.getAttribute_result();
        if (mapState.isEmpty()) {
            mapState.put(metroDevice.getOriginal_dpe(), metro);
            collector.collect(metroDevice);
        } else {
            if (mapState.contains(metroDevice.getOriginal_dpe())) {
                if (mapState.get(metroDevice.getOriginal_dpe()).equals(metro)) {
                } else {
                    mapState.put(metroDevice.getOriginal_dpe(), metro);
                    collector.collect(metroDevice);
                }
            } else {
                mapState.put(metroDevice.getOriginal_dpe(), metro);
                collector.collect(metroDevice);
            }
        }
    }
}
