package com.sunwin.metro.process;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.bean.ElectricityMeter;
import com.sunwin.metro.utils.DateUtils;
import com.sunwin.metro.utils.PerStringUtils;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import java.util.Date;
import java.util.HashMap;



/**
 * @program: sunwin_metro
 * @description: 对于窗口中的电表详情数据进行处理
 * @author: xuYJ
 * @create: 2021-04-25 16:03
 **/
public class ElectricityMeterProcess extends ProcessWindowFunction<Tuple4<String, String, String, Long>, ElectricityMeter, String, TimeWindow> {
    @Override
    public void process(String s, Context context, Iterable<Tuple4<String, String, String, Long>> elements, Collector<ElectricityMeter> out) throws Exception {
        ElectricityMeter electricityMeter = new ElectricityMeter();
        HashMap<String, Double> aVoltage = new HashMap<>(16);
        HashMap<String, Double> bVoltage = new HashMap<>(16);
        HashMap<String, Double> cVoltage = new HashMap<>(16);
        HashMap<String, Double> aCurrent = new HashMap<>(16);
        HashMap<String, Double> bCurrent = new HashMap<>(16);
        HashMap<String, Double> cCurrent = new HashMap<>(16);
        HashMap<String, Double> totalActivePower = new HashMap<>(16);
        HashMap<String, Double> aActivePower = new HashMap<>(16);
        HashMap<String, Double> bActivePower = new HashMap<>(16);
        HashMap<String, Double> cActivePower = new HashMap<>(16);
        HashMap<String, Double> totalNegativeActive = new HashMap<>(16);
        HashMap<String, Double> aNegativeActive = new HashMap<>(16);
        HashMap<String, Double> bNegativeActive = new HashMap<>(16);
        HashMap<String, Double> cNegativeActive = new HashMap<>(16);
        HashMap<String, Double> totalApparentPower = new HashMap<>(16);
        HashMap<String, Double> aApparentPower = new HashMap<>(16);
        HashMap<String, Double> bApparentPower = new HashMap<>(16);
        HashMap<String, Double> cApparentPower = new HashMap<>(16);
        HashMap<String, Double> totalPowerFactor = new HashMap<>(16);
        HashMap<String, Double> aPowerFactor = new HashMap<>(16);
        HashMap<String, Double> bPowerFactor = new HashMap<>(16);
        HashMap<String, Double> cPowerFactor = new HashMap<>(16);
        HashMap<String, Double> frequency = new HashMap<>(16);
        HashMap<String, Object> electricTotalActive = new HashMap<>(16);
        HashMap<String, Double> totalActive = new HashMap<>(16);
        HashMap<String, Object> meterNumber = new HashMap<>(16);
        HashMap<String, Object> electricStatus = new HashMap<>(16);

        elements.forEach(tuple3 -> {
            String f2 = tuple3.f2;
            JSONObject data = JSON.parseObject(JSON.parseObject(f2).getString("data"));
            JSONObject current = JSON.parseObject(data.getString("0"));
            String updateTime = String.valueOf(tuple3.f3);
            meterNumber.put(updateTime, current.getString("meter_number"));
            electricStatus.put(updateTime, current.getInteger("electric_status"));
            aVoltage.put(updateTime, current.getDouble("a_voltage"));
            bVoltage.put(updateTime, current.getDouble("b_voltage"));
            cVoltage.put(updateTime, current.getDouble("c_voltage"));
            aCurrent.put(updateTime, current.getDouble("a_current"));
            bCurrent.put(updateTime, current.getDouble("b_current"));
            cCurrent.put(updateTime, current.getDouble("c_current"));
            totalActivePower.put(updateTime, current.getDouble("total_active_power"));
            aActivePower.put(updateTime, current.getDouble("a_active_power"));
            bActivePower.put(updateTime, current.getDouble("b_active_power"));
            cActivePower.put(updateTime, current.getDouble("c_active_power"));
            totalNegativeActive.put(updateTime, current.getDouble("total_negative_active"));
            aNegativeActive.put(updateTime, current.getDouble("a_negative_active"));
            bNegativeActive.put(updateTime, current.getDouble("b_negative_active"));
            cNegativeActive.put(updateTime, current.getDouble("c_negative_active"));
            totalApparentPower.put(updateTime, current.getDouble("total_apparent_power"));
            aApparentPower.put(updateTime, current.getDouble("a_apparent_power"));
            bApparentPower.put(updateTime, current.getDouble("b_apparent_power"));
            cApparentPower.put(updateTime, current.getDouble("c_apparent_power"));
            totalPowerFactor.put(updateTime, current.getDouble("total_power_factor"));
            aPowerFactor.put(updateTime, current.getDouble("a_power_factor"));
            bPowerFactor.put(updateTime, current.getDouble("b_power_factor"));
            cPowerFactor.put(updateTime, current.getDouble("c_power_factor"));
            frequency.put(updateTime, current.getDouble("frequency"));
            electricTotalActive.put(updateTime, current.getDouble("electric_total_active"));
            totalActive.put(updateTime, current.getDouble("total_active"));
        });
        electricityMeter.setA_voltage(aVoltage);
        electricityMeter.setB_voltage(bVoltage);
        electricityMeter.setC_voltage(cVoltage);
        electricityMeter.setA_current(aCurrent);
        electricityMeter.setB_current(bCurrent);
        electricityMeter.setC_current(cCurrent);
        electricityMeter.setTotal_active_power(totalActivePower);
        electricityMeter.setA_active_power(aActivePower);
        electricityMeter.setB_active_power(bActivePower);
        electricityMeter.setC_active_power(cActivePower);
        electricityMeter.setTotal_negative_active(totalNegativeActive);
        electricityMeter.setA_negative_active(aNegativeActive);
        electricityMeter.setB_negative_active(bNegativeActive);
        electricityMeter.setC_negative_active(cNegativeActive);
        electricityMeter.setTotal_apparent_power(totalApparentPower);
        electricityMeter.setA_apparent_power(aApparentPower);
        electricityMeter.setB_apparent_power(bApparentPower);
        electricityMeter.setC_apparent_power(cApparentPower);
        electricityMeter.setTotal_power_factor(totalPowerFactor);
        electricityMeter.setA_power_factor(aPowerFactor);
        electricityMeter.setB_power_factor(bPowerFactor);
        electricityMeter.setC_power_factor(cPowerFactor);
        electricityMeter.setFrequency(frequency);
        electricityMeter.setElectric_total_active(Double.parseDouble(String.valueOf(PerStringUtils.getMaxValue(electricTotalActive))));
        electricityMeter.setTotal_active(totalActive);
        electricityMeter.setUpdate_time(PerStringUtils.getMaxKey(aVoltage));
        electricityMeter.setMeter_number(String.valueOf(PerStringUtils.getMaxValue(meterNumber)));
        electricityMeter.setElectric_status(Integer.parseInt(String.valueOf(PerStringUtils.getMaxValue(electricStatus))));
        out.collect(electricityMeter);
    }
}
