package com.sunwin.metro.process;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.util.Collector;
import org.apache.flink.util.OutputTag;

/**
 * @program: sunwin_metro
 * @description: 过滤异常仪表号（携带的数据为空的仪表号）
 * @author: xuYJ
 * @create: 2021-04-25 15:43
 **/
public class ErrorMeterProcess extends ProcessFunction<Tuple4<String, String, String, String>, Tuple3<String, String, String>> {
    private final OutputTag<Tuple3<String, String, String>> errorMeter;
    private final OutputTag<Tuple3<String, String, String>> waterMeter;
    private final OutputTag<Tuple3<String, String, String>> gasMeter;

    public ErrorMeterProcess(OutputTag<Tuple3<String, String, String>> errorMeter, OutputTag<Tuple3<String, String, String>> waterMeter, OutputTag<Tuple3<String, String, String>> gasMeter) {
        this.errorMeter = errorMeter;
        this.waterMeter = waterMeter;
        this.gasMeter = gasMeter;
    }

    @Override
    public void processElement(Tuple4<String, String, String, String> tuple4, Context context, Collector<Tuple3<String, String, String>> collector) throws Exception {
        try {
            JSONObject jsonObject = JSON.parseObject(tuple4.f3);
            JSONObject date = JSON.parseObject(jsonObject.getString("data"));
            if (jsonObject.getInteger("code") == 0
                    && "操作成功".equals(jsonObject.getString("msg"))
                    && !"".equals(date.getString("0"))
                    && !"".equals(date.getString("m15"))
                    && !"".equals(date.getString("m30"))
                    && !"".equals(date.getString("h1"))
                    && !"".equals(date.getString("h2"))) {
                if (Integer.parseInt(tuple4.f2) == 1) {
                    collector.collect(new Tuple3<>(tuple4.f0, tuple4.f1, tuple4.f3));
                } else if (Integer.parseInt(tuple4.f2) == 2) {
                    context.output(waterMeter, new Tuple3<>(tuple4.f0, tuple4.f1, tuple4.f3));
                } else {
                    context.output(gasMeter, new Tuple3<>(tuple4.f0, tuple4.f1, tuple4.f3));
                }
            } else {
                context.output(errorMeter, new Tuple3<>(tuple4.f0,tuple4.f1,tuple4.f2));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
