package com.sunwin.metro.process;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.utils.Sign;
import org.apache.flink.api.common.state.*;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.apache.flink.util.OutputTag;

/**
 * @program: sunwin_metro
 * @description: 地铁数据分组后进行的mapState+valueState去重
 * @author: xuYJ
 * @create: 2021-01-15 11:56
 **/
public class MetroKeyProcess extends KeyedProcessFunction<String, Tuple4<String, Integer, Long, String>, String> {
    private final OutputTag<Tuple2<String, Integer>> tag;
    private transient MapState<String, Integer> mapState;
    private transient ValueState<Integer> valueState;

    public MetroKeyProcess(OutputTag<Tuple2<String, Integer>> tag) {
        this.tag = tag;
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        StateTtlConfig ttlConfig = StateTtlConfig
                .newBuilder(Time.minutes(24 * 60))
                .setUpdateType(StateTtlConfig.UpdateType.OnCreateAndWrite)
                .setStateVisibility(StateTtlConfig.StateVisibility.NeverReturnExpired)
                .build();
        MapStateDescriptor<String, Integer> descriptor = new MapStateDescriptor<>("mapState", TypeInformation.of(String.class), TypeInformation.of(Integer.class));
        descriptor.enableTimeToLive(ttlConfig);
        ValueStateDescriptor<Integer> descriptor2 = new ValueStateDescriptor<>("valueState", TypeInformation.of(Integer.class));
        //设置过期时间
        descriptor2.enableTimeToLive(ttlConfig);
        mapState = getRuntimeContext().getMapState(descriptor);
        valueState = getRuntimeContext().getState(descriptor2);
        super.open(parameters);
        super.open(parameters);
    }

    @Override
    public void processElement(Tuple4<String, Integer, Long, String> tuple4, Context context, Collector<String> collector) throws Exception {
        String line = tuple4.f3;
        JSONObject jsonObject = JSON.parseObject(line);
        String key = context.getCurrentKey();
        String id = String.valueOf(Sign.getMd5(jsonObject.getString("compname") + jsonObject.getString("dpes")));
        if (mapState.isEmpty()) {
            mapState.put(id, 1);
            collector.collect(line);
        } else {
            if (null != mapState.get(id)) {
                Integer value = valueState.value();
                valueState.update(null == value ? 1 : value + 1);
                context.output(tag, new Tuple2<>(key, value));
            } else {
                mapState.put(id, 1);
                collector.collect(line);
            }
        }
    }
}
