package com.sunwin.metro.process;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.bean.ElectricityMeter;
import com.sunwin.metro.utils.DateUtils;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.Date;

/**
 * @program: sunwin_metro
 * @description: 固定整点时间提取数据输出
 * @author: xuYJ
 * @create: 2021-04-25 15:55
 **/
public class ScheduleTimeProcess extends KeyedProcessFunction<String, Tuple4<String, String, String, Long>, ElectricityMeter> {

    private final int time;

    public ScheduleTimeProcess(int time) {
        this.time = time;
    }

    @Override
    public void processElement(Tuple4<String, String, String, Long> value, Context ctx, Collector<ElectricityMeter> out) throws Exception {
        ElectricityMeter electricityMeter = new ElectricityMeter();
        String updateTime = DateUtils.dateToLongStringNoSep(new Date(value.f3));
        String min = updateTime.substring(10, 12);
        if (Integer.parseInt(min) % time == 0) {
            String input = value.f2;
            JSONObject data = JSON.parseObject(JSON.parseObject(input).getString("data"));
            JSONObject current = JSON.parseObject(data.getString("0"));
            electricityMeter.setElectric_status(current.getInteger("electric_status"));
            electricityMeter.setUpdate_time(updateTime.substring(0, 12));
            electricityMeter.setMeter_number(current.getString("meter_number"));
            electricityMeter.setElectric_total_active(current.getDouble("electric_total_active"));
            out.collect(electricityMeter);
        }
    }
}
