package com.sunwin.metro.service;

import com.sunwin.metro.bean.ElectricityMeter;
import com.sunwin.metro.bean.Meter;
import com.sunwin.metro.flatmap.MeterListFlatmap;
import com.sunwin.metro.key.MeterIdSelector;
import com.sunwin.metro.map.MeterTimeMap;
import com.sunwin.metro.map.MeterTimestampMap;
import com.sunwin.metro.process.ElectricityMeterProcess;
import com.sunwin.metro.process.ErrorMeterProcess;
import com.sunwin.metro.process.ScheduleTimeProcess;
import com.sunwin.metro.sink.impala.SinkToImpala;
import com.sunwin.metro.sink.impala.SinkToImpalaMeterValue;
import com.sunwin.metro.sink.mysql.SinkErrorMeterToMysql;
import com.sunwin.metro.sink.mysql.SinkNormalMeterToMysql;
import com.sunwin.metro.sink.mysql.SinkToMysql;
import com.sunwin.metro.watermark.MeterWaterLine;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.streaming.api.datastream.*;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.util.OutputTag;


/**
 * @program: sunwin_metro
 * @description: 能耗业务
 * @author: xuYJ
 * @create: 2021-04-14 09:54
 **/
public class EnergyConsumption {
    public static void process(SingleOutputStreamOperator<String> source, SingleOutputStreamOperator<Tuple4<String, String, String, String>> source1) {
        OutputTag<Tuple3<String, String, String>> errorMeter = new OutputTag<Tuple3<String, String, String>>("error_meter") {
        };
        OutputTag<Tuple3<String, String, String>> waterMeter = new OutputTag<Tuple3<String, String, String>>("waterMeter") {
        };
        OutputTag<Tuple3<String, String, String>> gasMeter = new OutputTag<Tuple3<String, String, String>>("gasMeter") {
        };
        OutputTag<Tuple4<String, String, String, Long>> lateOutputTag = new OutputTag<Tuple4<String, String, String, Long>>("late_data_output_tag") {
        };
        SingleOutputStreamOperator<Meter> flatMap = source.flatMap(new MeterListFlatmap());
        flatMap.addSink(new SinkToMysql());
        SingleOutputStreamOperator<Tuple3<String, String, String>> electricityMeter = source1.process(new ErrorMeterProcess(errorMeter, waterMeter, gasMeter));
        electricityMeter.addSink(new SinkNormalMeterToMysql());
        DataStream<Tuple3<String, String, String>> errorSide = electricityMeter.getSideOutput(errorMeter);
        errorSide.addSink(new SinkErrorMeterToMysql());
        DataStream<Tuple3<String, String, String>> waterSide = electricityMeter.getSideOutput(waterMeter);
        waterSide.addSink(new SinkNormalMeterToMysql());
        DataStream<Tuple3<String, String, String>> gasSide = electricityMeter.getSideOutput(gasMeter);
        gasSide.addSink(new SinkNormalMeterToMysql());
        //TODO：电表业务处理逻辑
        //把时间字符串转化成毫秒时间戳
        SingleOutputStreamOperator<Tuple4<String, String, String, Long>> map = electricityMeter.map(new MeterTimeMap());
        //设置水位线（0 延迟）
        SingleOutputStreamOperator<Tuple4<String, String, String, Long>> watermarks = map.assignTimestampsAndWatermarks(new MeterWaterLine(Time.seconds(0)));
        //根据仪表标识分组
        KeyedStream<Tuple4<String, String, String, Long>, String> keyedStream = watermarks.keyBy(new MeterIdSelector());
        //提取出字段中分钟  间隔5分钟（%5） 间隔15分钟（%15）
        keyedStream.process(new ScheduleTimeProcess(15))
                //输出到impala表 电表隔5分钟的实时度数
                .addSink(new SinkToImpalaMeterValue());

        //TODO：设定窗口时间+无水位线（不准延迟数据）
        SingleOutputStreamOperator<ElectricityMeter> process = keyedStream.timeWindow(Time.seconds(60 * 5))
                //设置数据延迟到达的时间
                .allowedLateness(Time.seconds(5 * 60))
                //侧视图收集迟到5分钟内的数据
                .sideOutputLateData(lateOutputTag)
                .process(new ElectricityMeterProcess());
        //电表业务数据处理后输出到impala
        process.addSink(new SinkToImpala());
        //窗口中电表延迟数据
        DataStream<Tuple4<String, String, String, Long>> sideOutput = process.getSideOutput(lateOutputTag);
        sideOutput.map(new MeterTimestampMap());
    }


}
