package com.sunwin.metro.service;

import com.sunwin.metro.bean.DeviceWarn;
import com.sunwin.metro.bean.MetroDevice;
import com.sunwin.metro.consumer.TopicKeyWordsConsumer;
import com.sunwin.metro.filter.MetroFilterMp;
import com.sunwin.metro.flatmap.MetroDevicePropertyFlatmap;
import com.sunwin.metro.key.DeviceKeySelector;
import com.sunwin.metro.key.DeviceNameKeySelector;
import com.sunwin.metro.key.MetroKeySelector;
import com.sunwin.metro.map.DeviceAlarmMap;
import com.sunwin.metro.map.DeviceMetricsMap;
import com.sunwin.metro.process.DeviceAlarmProcess;
import com.sunwin.metro.process.DeviceUnrepeated;
import com.sunwin.metro.process.MetroKeyProcess;
import com.sunwin.metro.sink.kudu.KuduSink;
import com.sunwin.metro.sink.kudu.connector.KuduTableInfo;
import com.sunwin.metro.watermark.DeviceWaterLine;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.util.OutputTag;


/**
 * @author xuYJ
 * @description: 消费kafka中地铁业务数据
 * @create: 2020-08-26 14:10
 * , BroadcastStream<Map<String, List<String>>> broadcast
 */
public class MetroService implements TopicKeyWordsConsumer {

    @Override
    public void process(DataStreamSource<Tuple4<String, Integer, Long, String>> input) {
        KuduTableInfo kuduTableInfo = KuduTableInfo.Builder.open("impala::urtdb.metro_system_record").build();
        KuduTableInfo tableName = KuduTableInfo.Builder.open("impala::urtdb.metro_warn_record").build();
        OutputTag<Tuple2<String, Integer>> tag = new OutputTag<Tuple2<String, Integer>>("heavy") {
        };
        SingleOutputStreamOperator<String> process = input
                .filter(new MetroFilterMp())
                .keyBy(new MetroKeySelector()).process(new MetroKeyProcess(tag));
        DataStream<Tuple2<String, Integer>> sideOutput = process.getSideOutput(tag);
        sideOutput.print();
        SingleOutputStreamOperator<MetroDevice> state = process.flatMap(new MetroDevicePropertyFlatmap())
                .map(new DeviceMetricsMap())
                .keyBy(new DeviceKeySelector())
                .process(new DeviceUnrepeated());
        state.addSink(new KuduSink<MetroDevice>("192.168.20.63:7051,192.168.20.64:7051,192.168.20.65:7051", kuduTableInfo, MetroDevice.class).withStrongConsistency());
        state.assignTimestampsAndWatermarks(new DeviceWaterLine(Time.milliseconds(1000)));
        SingleOutputStreamOperator<Tuple2<String, MetroDevice>> operator = state.keyBy(new DeviceNameKeySelector())
                .process(new DeviceAlarmProcess());
        operator
                .map(new DeviceAlarmMap())
                .addSink(new KuduSink<DeviceWarn>("192.168.20.63:7051,192.168.20.64:7051,192.168.20.65:7051", tableName, DeviceWarn.class).withStrongConsistency());
    }
}