package com.sunwin.metro.sink.impala;

import com.alibaba.fastjson.JSON;
import com.sunwin.metro.bean.ElectricityMeter;
import com.sunwin.metro.utils.MysqlUtils;
import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * @program: sunwin_metro
 * @description: 把能耗结果输出到impala
 * @author: xuYJ
 * @create: 2021-04-15 09:54
 **/
public class SinkToImpala extends RichSinkFunction<ElectricityMeter> {
    private Connection con = null;
    private ResultSet rs = null;
    private PreparedStatement ps = null;

    @Override
    public void open(Configuration parameters) throws Exception {
        con = MysqlUtils.getImpalaCon(new BasicDataSource());
        super.open(parameters);
    }

    @Override
    public void close() throws Exception {
        rs.close();
        ps.close();
        con.close();
        super.close();
    }

    @Override
    public void invoke(ElectricityMeter electricityMeter, Context context) throws Exception {
        String sql = " INSERT  INTO electricity_meter (" +
                "meter_number, update_time, electric_total_active, frequency, a_current, " +
                "b_current, c_current, a_voltage, b_voltage, " +
                "c_voltage, total_negative_active, a_negative_active, b_negative_active, " +
                "c_negative_active, total_power_factor, a_power_factor, b_power_factor, c_power_factor, total_apparent_power, a_apparent_power, " +
                "b_apparent_power, c_apparent_power, total_active_power, a_active_power, b_active_power, c_active_power, electric_status, total_active) " +
                "VALUES(?, ?, ?, ?, ?, ? , ? , ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )";
        ps = con.prepareStatement(sql);
        try {
            ps.setString(1, electricityMeter.getMeter_number());
            ps.setString(2, electricityMeter.getUpdate_time());
            ps.setDouble(3, electricityMeter.getElectric_total_active());
            ps.setObject(4, JSON.toJSONString(mapSort(electricityMeter.getFrequency())));
            ps.setObject(5, JSON.toJSONString(mapSort(electricityMeter.getA_current())));
            ps.setObject(6, JSON.toJSONString(mapSort(electricityMeter.getB_current())));
            ps.setObject(7, JSON.toJSONString(mapSort(electricityMeter.getC_current())));
            ps.setObject(8, JSON.toJSONString(mapSort(electricityMeter.getA_voltage())));
            ps.setObject(9, JSON.toJSONString(mapSort(electricityMeter.getB_voltage())));
            ps.setObject(10, JSON.toJSONString(mapSort(electricityMeter.getC_voltage())));
            ps.setObject(11, JSON.toJSONString(mapSort(electricityMeter.getTotal_negative_active())));
            ps.setObject(12, JSON.toJSONString(mapSort(electricityMeter.getA_negative_active())));
            ps.setObject(13, JSON.toJSONString(mapSort(electricityMeter.getB_negative_active())));
            ps.setObject(14, JSON.toJSONString(mapSort(electricityMeter.getC_negative_active())));
            ps.setObject(15, JSON.toJSONString(mapSort(electricityMeter.getTotal_power_factor())));
            ps.setObject(16, JSON.toJSONString(mapSort(electricityMeter.getA_power_factor())));
            ps.setObject(17, JSON.toJSONString(mapSort(electricityMeter.getB_power_factor())));
            ps.setObject(18, JSON.toJSONString(mapSort(electricityMeter.getC_power_factor())));
            ps.setObject(19, JSON.toJSONString(mapSort(electricityMeter.getTotal_apparent_power())));
            ps.setObject(20, JSON.toJSONString(mapSort(electricityMeter.getA_apparent_power())));
            ps.setObject(21, JSON.toJSONString(mapSort(electricityMeter.getB_apparent_power())));
            ps.setObject(22, JSON.toJSONString(mapSort(electricityMeter.getC_apparent_power())));
            ps.setObject(23, JSON.toJSONString(mapSort(electricityMeter.getTotal_active_power())));
            ps.setObject(24, JSON.toJSONString(mapSort(electricityMeter.getA_active_power())));
            ps.setObject(25, JSON.toJSONString(mapSort(electricityMeter.getB_active_power())));
            ps.setObject(26, JSON.toJSONString(mapSort(electricityMeter.getC_active_power())));
            ps.setInt(27, electricityMeter.getElectric_status());
            ps.setObject(28, JSON.toJSONString(mapSort(electricityMeter.getTotal_active())));
            ps.addBatch();
            ps.executeBatch();
            con.commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Map<String, Double> mapSort(Map<String, Double> map) {
        Map<String, Double> mapSort = new TreeMap<>(Comparator.reverseOrder());
        mapSort.putAll(map);
        return mapSort;
    }

    public String mapSortValue(Map<String, Double> map) {
        Map<String, Double> mapSort = new TreeMap<>(Comparator.reverseOrder());
        mapSort.putAll(map);
        return String.valueOf(mapSort.values());
    }


}
