package com.sunwin.metro.sink.impala;

import com.alibaba.fastjson.JSON;
import com.sunwin.metro.bean.ElectricityMeter;
import com.sunwin.metro.utils.MysqlUtils;
import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Comparator;
import java.util.Map;
import java.util.TreeMap;

/**
 * @program: sunwin_metro
 * @description: 把能耗结果输出到impala
 * @author: xuYJ
 * @create: 2021-04-15 09:54
 **/
public class SinkToImpalaMeterValue extends RichSinkFunction<ElectricityMeter> {
    private Connection con = null;
    private ResultSet rs = null;
    private PreparedStatement ps = null;

    @Override
    public void open(Configuration parameters) throws Exception {
        con = MysqlUtils.getImpalaCon(new BasicDataSource());
        super.open(parameters);
    }

    @Override
    public void close() throws Exception {
        rs.close();
        ps.close();
        con.close();
        super.close();
    }

    @Override
    public void invoke(ElectricityMeter electricityMeter, Context context) throws Exception {
        String sql = " INSERT  INTO electricity_meter_value (" +
                "meter_number, update_time, electric_total_active, electric_status) " +
                "VALUES(?, ?, ?, ? )";
        ps = con.prepareStatement(sql);
        try {
            ps.setString(1, electricityMeter.getMeter_number());
            ps.setString(2, electricityMeter.getUpdate_time());
            ps.setDouble(3, electricityMeter.getElectric_total_active());
            ps.setInt(4, electricityMeter.getElectric_status());
            ps.addBatch();
            ps.executeBatch();
            con.commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
