package com.sunwin.metro.source.http;


import com.chinaway.http.client.HttpClientResponse;
import com.sunwin.metro.bean.Meter;
import com.sunwin.metro.utils.MysqlUtils;
import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * @program: sunwin_metro
 * @description: http数据源 - 10s采集一次
 * @author: xuYJ
 * @create: 2021-04-14 09:37
 **/
public class HttpMeterMsg extends RichSourceFunction<Tuple4<String, String, String, String>> {
    private HttpClientResponse<String> httpClient;
    private Map<String, String> params;
    private Connection con;
    private ResultSet rs;
    private PreparedStatement pStatement;

    @Override
    public void open(Configuration parameters) throws Exception {
        params = new HashMap<>(16);
        httpClient = new HttpClientResponse<>();
        con = MysqlUtils.getMysqlCon(new BasicDataSource());
        super.open(parameters);
    }

    @Override
    public void close() throws Exception {
        super.close();
        rs.close();
        pStatement.close();
        con.close();
        super.close();
    }

    @Override
    public void run(SourceContext<Tuple4<String, String, String, String>> sourceContext) throws Exception {
        String sql = "select id, meter_number, meter_type from meter";
        while (true) {
            List<Meter> meters = new ArrayList<>();
            try {
                pStatement = con.prepareStatement(sql);
                rs = pStatement.executeQuery();
                while (rs.next()) {
                    meters.add(new Meter(rs.getString("id"),rs.getString("meter_number"),rs.getInt("meter_type")));
                }
            } catch (Exception e) {
                e.printStackTrace();
                throw e;
            }
            List<Meter> newList = meters.stream().filter(distinctByKey(Meter::getMeterNumber)).collect(Collectors.toList());
            for (Meter meter : newList) {
                params.put("meterNumber", meter.getMeterNumber());
                String result = httpClient.getForString("http://192.168.20.134:8083//home/forTimes", params, new HashMap<>(16));
                sourceContext.collect(new Tuple4<>(meter.getId(), meter.getMeterNumber(), String.valueOf(meter.getMeterType()), result));
            }
            Thread.sleep(10000L);
        }
    }

    @Override
    public void cancel() {

    }

    private static <T> Predicate<T> distinctByKey(Function<? super T, Object> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>(16);
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }


}
