package com.sunwin.metro.source.mysql;

import com.sunwin.metro.utils.PerStringUtils;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @program: boryou-flink
 * @description: 获取mysql关键字自定义流
 * @author: xuYJ
 * @create: 2020-10-19 15:42
 **/

public class MysqlSource extends RichSourceFunction<Map<String, List<String>>> {
    public static final Logger logger = LoggerFactory.getLogger(MysqlSource.class);
    private volatile boolean isRunning = true;

    private final Long time;

    public MysqlSource(Long time) {
        this.time = time;
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
    }

    @Override
    public void run(SourceContext<Map<String,List<String>>> ctx) throws Exception {
        while (isRunning) {
            Map<String,List<String>> keyWords = new HashMap<>(16);
            try {
                List<String> words = MysqlData.getWords("metro_device_all");
                if (!PerStringUtils.isNullOrEmpty(words)) {
                    keyWords.put("devices",words);
                    ctx.collect(keyWords);
                }
            }catch (Exception e){
                e.printStackTrace();
            }
            Thread.sleep(time);
        }
    }

    @Override
    public void cancel() {
        isRunning = false;
    }
}
