package com.sunwin.metro.source.mysql;

import com.sunwin.metro.bean.DeviceName;
import com.sunwin.metro.utils.PerStringUtils;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @program: boryou-flink
 * @description: 获取mysql关键字自定义流
 * @author: xuYJ
 * @create: 2020-10-19 15:42
 **/

public class MysqlSourceTime extends RichSourceFunction<Map<String, List<String>>> {
    public static final Logger logger = LoggerFactory.getLogger(MysqlSourceTime.class);
    private volatile boolean isRunning = true;

    private final Long time;

    public MysqlSourceTime(Long time) {
        this.time = time;
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
    }

    @Override
    public void run(SourceContext<Map<String,List<String>>> ctx) throws Exception {
        long date=20000000000000L;
        while (isRunning) {
            ArrayList<String> out = new ArrayList<>();
            Map<String,List<String>> keyWords = new HashMap<>(16);
            try {
                List<DeviceName> devices = MysqlData.getWordsWithTime("metro_device_all",date);
                for (DeviceName device : devices) {
                    out.add(device.getDeviceName());
                    if(device.getSendTime()>date){
                        date=device.getSendTime();
                    }
                }
                if (!PerStringUtils.isNullOrEmpty(out)) {
                    keyWords.put("devices",out);
                    ctx.collect(keyWords);
                }
            }catch (Exception e){
                e.printStackTrace();
            }
            Thread.sleep(time);
        }
    }

    @Override
    public void cancel() {
        isRunning = false;
    }
}
