package com.sunwin.metro.start;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.bean.message.EquipmentWarn;
import com.sunwin.metro.sink.impala.SinkToImpalaWarn;
import com.sunwin.metro.sink.kudu.connector.KuduTableInfo;
import com.sunwin.metro.source.mysql.message.QueryEquipment;
import com.sunwin.metro.utils.DateUtils;
import org.apache.flink.api.common.JobExecutionResult;
import org.apache.flink.api.common.state.*;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.java.tuple.Tuple;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.metrics.Counter;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.streaming.api.functions.source.RichSourceFunction;
import org.apache.flink.util.Collector;
import java.util.*;

/**
 * @program: sunwin_metro
 * @description: 模拟数据
 * @author: xuYJ
 * @create: 2021-05-31 11:33
 **/
public class StartMessage {
    public static void main(String[] args) throws Exception {
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        StreamPro.envPro(env, args[0]);
        DataStreamSource<Tuple2<String, String>> source = env.addSource(new RichSourceFunction<Tuple2<String, String>>() {
            @Override
            public void run(SourceContext<Tuple2<String, String>> ctx) throws Exception {
                JSONArray array = QueryEquipment.getId();
                while (true) {
                    int i = new Random().nextInt(20) + 1;
                    JSONArray jsonArray = getArray(array, i);
                    for (Object o : jsonArray) {
                        ctx.collect(new Tuple2<>("key", o.toString()));
                    }
                    Thread.sleep(1000 * 60 * i);
                }
            }
            @Override
            public void cancel() {
            }
        });
        SingleOutputStreamOperator<EquipmentWarn> result = source.keyBy(0).process(new KeyedProcessFunction<Tuple, Tuple2<String, String>, EquipmentWarn>() {
            private transient MapState<String, JSONArray> mapState;
            private transient MapState<String, String> mapState2;
            private transient ListState<EquipmentWarn> listState;
            private transient Counter ct;
            private transient ValueState<Long> valueState;

            @Override
            public void open(Configuration parameters) throws Exception {
                MapStateDescriptor<String, JSONArray> descriptor = new MapStateDescriptor<>("mapState", TypeInformation.of(String.class), TypeInformation.of(JSONArray.class));
                MapStateDescriptor<String, String> descriptor2 = new MapStateDescriptor<>("mapState2", TypeInformation.of(String.class), TypeInformation.of(String.class));
                ValueStateDescriptor<Long> descriptor4 = new ValueStateDescriptor<>("count", TypeInformation.of(Long.class));
                mapState = getRuntimeContext().getMapState(descriptor);
                mapState2 = getRuntimeContext().getMapState(descriptor2);
                valueState = getRuntimeContext().getState(descriptor4);
                ListStateDescriptor<EquipmentWarn> descriptor3 = new ListStateDescriptor<>("listState", TypeInformation.of(EquipmentWarn.class));
                listState = getRuntimeContext().getListState(descriptor3);
                ct = getRuntimeContext().getMetricGroup().counter("ct");
                super.open(parameters);
            }

            @Override
            public void close() throws Exception {
                super.close();
                QueryEquipment.closeCon();
            }

            @Override
            public void processElement(Tuple2<String, String> value, Context ctx, Collector<EquipmentWarn> out) throws Exception {
                ct.inc();
                long count = ct.getCount();
                valueState.update(count);
                if (mapState.isEmpty()) {
                    mapState.putAll(QueryEquipment.getWarnMessage());
                }
                if (mapState2.isEmpty()) {
                    mapState2.putAll(QueryEquipment.getEquipmentName());
                    QueryEquipment.closeCon();
                }
                ArrayList<String> list = new ArrayList<>();
                list.add("m_0");
                list.add("m_30");
                list.add("h_1");
                list.add("h_2");
                list.add("h_4");
                list.add("h_8");
                EquipmentWarn equipmentWarn = new EquipmentWarn();
                JSONObject jsonObject = JSONObject.parseObject(value.f1);
                String equipmentId = String.valueOf(jsonObject.getInteger("equipment_id"));
                equipmentWarn.setCurrent_time(DateUtils.dateToShortStringNoSep(new Date()));
                equipmentWarn.setEquipment_code(jsonObject.getInteger("equipment_code"));
                equipmentWarn.setEquipment_name(mapState2.get(equipmentId));
                equipmentWarn.setStation(jsonObject.getInteger("station"));
                equipmentWarn.setFlag("true");
                equipmentWarn.setLine(jsonObject.getInteger("line"));
                equipmentWarn.setWarn_time(list.get(new Random().nextInt(list.size())));
                JSONArray array = mapState.get(equipmentId);
                equipmentWarn.setWarn_type(array.getString(new Random().nextInt(array.size())));
                GregorianCalendar calendar = new GregorianCalendar();
                int hour = calendar.get(Calendar.HOUR_OF_DAY);
                JSONArray jsonArray = new JSONArray();
                Iterable<EquipmentWarn> equipmentWarns = listState.get();
                equipmentWarns.forEach(jsonArray::add);
                if (hour == 1 && jsonArray.size() > 200) {
                    List<EquipmentWarn> warns = getArray(jsonArray, 200).toJavaList(EquipmentWarn.class);
                    for (EquipmentWarn warn : warns) {
                        out.collect(warn);
                    }
                    listState.clear();
                }
                listState.add(equipmentWarn);
                out.collect(equipmentWarn);
                System.out.println("状态变量中的集合数目" + jsonArray.size());
            }
        });
        result.addSink(new SinkToImpalaWarn());
    }
    public static JSONArray getArray(JSONArray list, int count) {
        JSONArray backList;
        backList = new JSONArray();
        Random random = new Random();
        int backSum = Math.min(list.size(), count);
        for (int i = 0; i < backSum; i++) {
            int target = random.nextInt(list.size());
            backList.add(list.get(target));
            list.remove(target);
        }
        return backList;
    }
}
