package com.sunwin.metro.start;


import com.alibaba.fastjson.JSONObject;
import com.sunwin.metro.bean.train.*;
import com.sunwin.metro.sink.kudu.KuduSink;
import com.sunwin.metro.sink.kudu.connector.KuduTableInfo;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.LocalStreamEnvironment;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.util.Collector;
import org.apache.flink.util.OutputTag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.sunwin.metro.start.StartService.getInput;

/**
 * @author xuYJ
 * @description: 清洗3号线设备属性数据
 * @create: 2020-08-26 14:10
 */
public class StartMetro {
    public static final Logger logger = LoggerFactory.getLogger(StartMetro.class);

    public static void main(String[] args) throws Exception {
        KuduTableInfo kuduTableInfo4 = KuduTableInfo.Builder.open("impala::urtdb.ups_bas").build();
        LocalStreamEnvironment env = StreamExecutionEnvironment.createLocalEnvironment();

        OutputTag<BasTef> tef = new OutputTag<BasTef>("BasTef") {
        };
        OutputTag<BasTvf> tvf = new OutputTag<BasTvf>("BasTvf") {
        };
        OutputTag<UpsBas> ups = new OutputTag<UpsBas>("UpsBasStation") {
        };
        OutputTag<UpsXjy> ups2 = new OutputTag<UpsXjy>("UpsXjy") {
        };
        OutputTag<Ups> ups3 = new OutputTag<Ups>("Ups") {
        };
        StreamPro.envPro(env, "BAS_TEF");
        DataStreamSource<Tuple4<String, Integer, Long, String>> source = getInput(env, "ups_bas", "bas1");
        SingleOutputStreamOperator<String> process = source.process(new ProcessFunction<Tuple4<String, Integer, Long, String>, String>() {
            @Override
            public void processElement(Tuple4<String, Integer, Long, String> tuple4, Context ctx, Collector<String> out) throws Exception {
                JSONObject json = JSONObject.parseObject(tuple4.f3);
                String name = json.getString("name").trim().split(":")[2].trim();
                String station = json.getString("name").trim().split(":")[1].trim();
                if (json.getString("name").contains("BAS_TEF")) {
                    ctx.output(tef, new BasTef(json.getString("DI:MOvL"), json.getIntValue("AI:FltNo"), json.getString("DI:AFlagSt"), json.getIntValue("AI:WindTA"), json.getIntValue("AI:WindTB"), json.getIntValue("AI:WindTC"), json.getIntValue("AI:Y_Vibr"), json.getIntValue("AO:SPTm"), json.getIntValue("AO:FSet"), json.getString("DI:Fail"), json.getIntValue("AI:FBak"), json.getString("DI:VFFlt"), json.getString("DI:sEnableSt"), json.getString("DI:AOvT"), json.getString("DI:Vibr"), json.getIntValue("AI:FAxleT"), json.getIntValue("AI:X_Vibr"), json.getString("DI:FRun"), json.getIntValue("AO:KpTm"), json.getString("DO:AFlagSt"), json.getString("DI:Stp"), json.getString("name"), json.getString("DI:VRun"), json.getIntValue("AI:RAxleT"), json.getDouble("AI:TAcc"), json.getString("DO:FRes"), json.getString("DI:VFBas"), json.getString("DO:FST"), json.getString("DI:CommOKSt"), json.getString("DO:sEnableSt"), json.getString("stime"), json.getString("DO:sFRestSt"), json.getString("DO:SP"), json.getIntValue("AI:PvoltC"), json.getString("DO:ST"), json.getIntValue("AI:PvoltA"), json.getIntValue("AI:PvoltB"), json.getString("DI:WOvT"), json.getString("DI:Run"), json.getIntValue("AI:Comm_Word"), json.getString("DI:Hang"), json.getString("DI:SFlagSt"), json.getString("DO:sUnableSt"), json.getString("DI:PFVF"), json.getString("DO:Hang"), json.getString("DO:SFlagSt"), json.getString("DO:NHang"), json.getString("DO:sRTRestSt"), json.getIntValue("AO:KiTm"), json.getString("DI:LR"), json.getIntValue("AI:PcurrA"), json.getIntValue("AI:PcurrC"), json.getIntValue("AI:PcurrB")));
                } else if (json.getString("name").contains("BAS_TVF")) {
                    ctx.output(tvf, new BasTvf(json.getIntValue("AI:TAcc"), json.getString("DO:FRes"), json.getString("DO:FireSP"), json.getString("DI:MOvL"), json.getIntValue("AI:FltNo"), json.getString("DO:FST"), json.getString("DI:ByPass"), json.getString("DI:CommOKSt"), json.getString("DI:AFlagSt"), json.getString("stime"), json.getString("DO:SP"), json.getString("DO:sFRestSt"), json.getString("DI:WOvL"), json.getIntValue("AI:PvoltC"), json.getIntValue("AI:PvoltA"), json.getString("DO:FireRST"), json.getIntValue("AI:PvoltB"), json.getIntValue("AI:WindTA"), json.getString("DO:FireFST"), json.getIntValue("AI:WindTB"), json.getIntValue("AI:WindTC"), json.getIntValue("AI:Y_Vibr"), json.getString("DI:Fail"), json.getString("DI:VFFlt"), json.getString("DI:AOvT"), json.getString("DI:SFlagSt"), json.getString("DI:Hang"), json.getString("DI:Vibr"), json.getString("DO:Hang"), json.getString("DO:SFlagSt"), json.getString("DI:SFBas"), json.getIntValue("AI:FAxleT"), json.getIntValue("AI:X_Vibr"), json.getString("DI:FRun"), json.getString("DI:Stp"), json.getString("DO:AFlagSt"), json.getString("DO:NHang"), json.getString("name"), json.getString("DO:sRTRestSt"), json.getString("DO:RST"), json.getString("DI:LR"), json.getIntValue("AI:PcurrA"), json.getIntValue("AI:RAxleT"), json.getIntValue("AI:PcurrC"), json.getIntValue("AI:PcurrB"), json.getString("DI:RRun")
                    ));
                } else if (name.contains("UPS_BAS")) {
                    ctx.output(ups, new UpsBas(
                            name,
                            station,
                            json.getString("stime"),
                            json.getIntValue("AI:BVP"),
                            json.getIntValue("AI:OP"),
                            json.getIntValue("AI:IV"),
                            json.getIntValue("AI:ROV"),
                            json.getIntValue("AI:IFV"),
                            json.getIntValue("AI:OV"),
                            json.getIntValue("AI:BV"),
                            json.getIntValue("AI:Temp"),
                            json.getIntValue("AI:ROFre"),
                            json.getIntValue("AI:RBV"),
                            json.getIntValue("AI:ROC"),
                            json.getIntValue("AI:IFre"),
                            json.getString("DI:BVLL"),
                            json.getString("DI:SDActive"),
                            json.getString("DI:UPSType"),
                            json.getString("DI:SYSTesting"),
                            json.getString("DI:CommOK"),
                            json.getString("DI:UPSFault"),
                            json.getString("DI:UPSMode"),
                            json.getString("DI:VoiceOpen"),
                            json.getString("DI:ESFault")
                    ));
                } else if (name.contains("UPS_XJY")) {
                    ctx.output(ups2,new UpsXjy(
                            json.getString("stime"),
                            name,
                            station,
                            json.getIntValue("AI:R1"),
                            json.getIntValue("AI:R2"),
                            json.getIntValue("AI:R3"),
                            json.getIntValue("AI:R4"),
                            json.getIntValue("AI:R5"),
                            json.getIntValue("AI:R6"),
                            json.getIntValue("AI:R7"),
                            json.getIntValue("AI:R8"),
                            json.getIntValue("AI:R9"),
                            json.getIntValue("AI:R10"),
                            json.getIntValue("AI:R11"),
                            json.getIntValue("AI:R12"),
                            json.getIntValue("AI:R13"),
                            json.getIntValue("AI:R14"),
                            json.getIntValue("AI:R15"),
                            json.getIntValue("AI:R16"),
                            json.getIntValue("AI:R17"),
                            json.getIntValue("AI:R18"),
                            json.getIntValue("AI:R19"),
                            json.getIntValue("AI:R20"),
                            json.getIntValue("AI:R21"),
                            json.getIntValue("AI:R22"),
                            json.getIntValue("AI:R23"),
                            json.getIntValue("AI:R24"),
                            json.getIntValue("AI:R25"),
                            json.getIntValue("AI:R26"),
                            json.getIntValue("AI:R27"),
                            json.getIntValue("AI:R28"),
                            json.getIntValue("AI:R29"),
                            json.getIntValue("AI:R30"),
                            json.getIntValue("AI:U1"),
                            json.getIntValue("AI:U2"),
                            json.getIntValue("AI:U3"),
                            json.getIntValue("AI:U4"),
                            json.getIntValue("AI:U5"),
                            json.getIntValue("AI:U6"),
                            json.getIntValue("AI:U7"),
                            json.getIntValue("AI:U8"),
                            json.getIntValue("AI:U9"),
                            json.getIntValue("AI:U10"),
                            json.getIntValue("AI:U11"),
                            json.getIntValue("AI:U12"),
                            json.getIntValue("AI:U13"),
                            json.getIntValue("AI:U14"),
                            json.getIntValue("AI:U15"),
                            json.getIntValue("AI:U16"),
                            json.getIntValue("AI:U17"),
                            json.getIntValue("AI:U18"),
                            json.getIntValue("AI:U19"),
                            json.getIntValue("AI:U20"),
                            json.getIntValue("AI:U21"),
                            json.getIntValue("AI:U22"),
                            json.getIntValue("AI:U23"),
                            json.getIntValue("AI:U24"),
                            json.getIntValue("AI:U25"),
                            json.getIntValue("AI:U26"),
                            json.getIntValue("AI:U27"),
                            json.getIntValue("AI:U28"),
                            json.getIntValue("AI:U29"),
                            json.getIntValue("AI:U30"),
                            json.getIntValue("AI:T1"),
                            json.getIntValue("AI:T2"),
                            json.getIntValue("AI:T3"),
                            json.getIntValue("AI:T4"),
                            json.getIntValue("AI:T5"),
                            json.getIntValue("AI:T6"),
                            json.getIntValue("AI:T7"),
                            json.getIntValue("AI:T8"),
                            json.getIntValue("AI:T9"),
                            json.getIntValue("AI:T10"),
                            json.getIntValue("AI:T11"),
                            json.getIntValue("AI:T12"),
                            json.getIntValue("AI:T13"),
                            json.getIntValue("AI:T14"),
                            json.getIntValue("AI:T15"),
                            json.getIntValue("AI:T16"),
                            json.getIntValue("AI:T17"),
                            json.getIntValue("AI:T18"),
                            json.getIntValue("AI:T19"),
                            json.getIntValue("AI:T20"),
                            json.getIntValue("AI:T21"),
                            json.getIntValue("AI:T22"),
                            json.getIntValue("AI:T23"),
                            json.getIntValue("AI:T24"),
                            json.getIntValue("AI:T25"),
                            json.getIntValue("AI:T26"),
                            json.getIntValue("AI:T27"),
                            json.getIntValue("AI:T28"),
                            json.getIntValue("AI:T29"),
                            json.getIntValue("AI:T30"),
                            json.getIntValue("AI:WD"),
                            json.getIntValue("AI:ZDY"),
                            json.getIntValue("AI:ZDL")
                    ) );
                }else {
                    ctx.output(ups3,new Ups(
                            json.getString("stime"),
                            name,
                            station,
                            json.getIntValue("AI:DP17"),
                            json.getIntValue("AI:DP18"),
                            json.getIntValue("AI:DP19"),
                            json.getIntValue("AI:DP20"),
                            json.getIntValue("AI:DP21"),
                            json.getIntValue("AI:DP22"),
                            json.getIntValue("AI:DP23"),
                            json.getIntValue("AI:DP24"),
                            json.getIntValue("AI:DP25"),
                            json.getIntValue("AI:DP26"),
                            json.getIntValue("AI:DP27"),
                            json.getIntValue("AI:DP28"),
                            json.getIntValue("AI:DP29"),
                            json.getIntValue("AI:DP32"),
                            json.getIntValue("AI:DP34"),
                            json.getIntValue("AI:DP35"),
                            json.getIntValue("AI:DP36"),
                            json.getString("DI:DP17"),
                            json.getString("DI:DP18"),
                            json.getString("DI:DP19"),
                            json.getString("DI:DP20"),
                            json.getString("DI:DP21"),
                            json.getString("DI:DP22"),
                            json.getString("DI:DP23"),
                            json.getString("DI:DP24"),
                            json.getString("DI:DP25"),
                            json.getString("DI:DP26"),
                            json.getString("DI:DP27"),
                            json.getString("DI:DP29"),
                            json.getString("DI:DP30"),
                            json.getString("DI:DP31"),
                            json.getString("DI:DP32"),
                            json.getString("DI:DP33"),
                            json.getString("DI:DP34"),
                            json.getString("DI:DP35"),
                            json.getString("DI:DP36")
                            ));
                }
            }
        }
    );
        process.getSideOutput(ups).addSink(new KuduSink<UpsBas>("192.168.20.63:7051,192.168.20.64:7051,192.168.20.65:7051", kuduTableInfo4,  UpsBas.class).withStrongConsistency());
        env.execute();
}
}
