package com.sunwin.metro.start;


import com.sunwin.metro.config.ConfigFileHandler;
import com.sunwin.metro.consumer.CustomKafkaDeserializationSchema;
import com.sunwin.metro.service.MetroService;
import com.sunwin.metro.service.WriteMetroTxt;
import com.sunwin.metro.utils.DateUtils;
import com.sunwin.metro.utils.MysqlUtils;
import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.commons.io.FileUtils;
import org.apache.flink.api.common.restartstrategy.RestartStrategies;
import org.apache.flink.api.java.tuple.Tuple4;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.contrib.streaming.state.RocksDBStateBackend;
import org.apache.flink.streaming.api.CheckpointingMode;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.apache.flink.streaming.api.functions.source.SourceFunction;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

/**
 * @author xuYJ
 * @description: 数据消费者
 * @create: 2020-08-26 14:10
 */
public class StartService {
    public static final Logger logger = LoggerFactory.getLogger(StartService.class);

    public static void main(String[] args) throws Exception {

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setRestartStrategy(RestartStrategies.fixedDelayRestart(10, org.apache.flink.api.common.time.Time.of(5, TimeUnit.SECONDS)));
        env.setStateBackend(new RocksDBStateBackend(ConfigFileHandler.getCheckPoint() + args[0] + "/", true).getCheckpointBackend());
        env.enableCheckpointing(10000 * 6);
        env.getCheckpointConfig().setMinPauseBetweenCheckpoints(10000 * 3);
        env.getCheckpointConfig().setCheckpointingMode(CheckpointingMode.EXACTLY_ONCE);
        env.getCheckpointConfig().setCheckpointTimeout(60000 * 10);
        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(100);
        env.getCheckpointConfig().setMaxConcurrentCheckpoints(1);
        env.getCheckpointConfig().enableExternalizedCheckpoints(CheckpointConfig.ExternalizedCheckpointCleanup.RETAIN_ON_CANCELLATION);
        switch (args[0]) {
            case "metro":
                new MetroService().process(getInput(env, ConfigFileHandler.getMetroTopicName(), ConfigFileHandler.getMetroId()));
                env.execute("metro");
                break;
            case "writeTxt":
                new WriteMetroTxt().process(getInput(env, ConfigFileHandler.getMetroTopicName(), ConfigFileHandler.getMetroId()));
                env.execute("writeTxt");
                break;
            case "TCMS-info":
                DataStreamSource<String> source5 = env.setParallelism(1).addSource(new SourceFunction<String>() {
                    private Boolean running = true;
                    @Override
                    public void run(SourceContext<String> ctx) throws Exception {
                        List<String> list = FileUtils.readLines(new File("C:\\Users\\daoda\\Desktop\\地铁项目\\TCMS曲线数据4小时.txt"));
                        while (running) {
                            for (int i = 0; i < list.size(); i++) {
                                ctx.collect(list.get(i));
                                Thread.sleep(500L);
                            }
                        }
                    }

                    @Override
                    public void cancel() {
                        running = false;
                    }
                });
                source5.addSink(new RichSinkFunction<String>() {
                    private PreparedStatement ps;
                    private Connection connection;

                    @Override
                    public void open(Configuration parameters) throws Exception {
                        super.open(parameters);
                        BasicDataSource dataSource = new BasicDataSource();
                        connection = MysqlUtils.getMysqlCon(dataSource);
                    }

                    @Override
                    public void close() throws Exception {
                        super.close();
                        super.close();
                        if (connection != null) {
                            connection.close();
                        }
                        if (ps != null) {
                            ps.close();
                        }
                    }

                    @Override
                    public void invoke(String value, Context context) throws Exception {
                        try {
                            String sql = "REPLACE into meter_single(collect_id, collect_time, atc_to_signal, atc_pre_arrival_signal, atc_away_signal, atc_next_id, " +
                                    "atc_current_id, atc_end_id, tmc2_next_id, tmc2_current_id, tmc2_end_id, tmc2_start_id " +
                                    ") values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);";
                            ps = connection.prepareStatement(sql);
                            String[] result = value.trim().split(",");
                            ps.setString(1, "result");
                            ps.setString(2, DateUtils.localDateTimeToStringWithFormat(LocalDateTime.now(), "HH:mm:ss:SSS"));
                            ps.setDouble(3, Double.parseDouble(result[1]));
                            ps.setDouble(4, Double.parseDouble(result[2]));
                            ps.setDouble(5, Double.parseDouble(result[3]));
                            ps.setDouble(6, Double.parseDouble(result[4]));
                            ps.setDouble(7, Double.parseDouble(result[5]));
                            ps.setDouble(8, Double.parseDouble(result[6]));
                            ps.setDouble(9, Double.parseDouble(result[7]));
                            ps.setDouble(10, Double.parseDouble(result[8]));
                            ps.setDouble(11, Double.parseDouble(result[9]));
                            ps.setDouble(12, Double.parseDouble(result[10]));
                            ps.addBatch();
                            ps.executeBatch();
                            connection.commit();
                        } catch (Exception e) {
                            e.printStackTrace();
                            if (connection != null) {
                                connection.close();
                            }
                            if (ps != null) {
                                ps.close();
                            }
                            BasicDataSource dataSource = new BasicDataSource();
                            connection = MysqlUtils.getMysqlCon(dataSource);
                        }
                    }
                });
                env.execute("TCMS-info");
                break;
            default:
                break;
        }
    }

    public static DataStreamSource<Tuple4<String, Integer, Long, String>> getInput(StreamExecutionEnvironment env, String topicName, String groupId) {
        Properties properties = new Properties();
        properties.setProperty("bootstrap.servers", ConfigFileHandler.getKafkaAddress());
        properties.setProperty("group.id", groupId);
        properties.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        properties.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        FlinkKafkaConsumer<Tuple4<String, Integer, Long, String>> consumer = new FlinkKafkaConsumer<>(topicName, new CustomKafkaDeserializationSchema(), properties);
        consumer.setStartFromEarliest();
        return env.addSource(consumer);
    }
}
