package com.sunwin.metro.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;


public class DateUtils {

    public static Date stringToDateWithFormat(String str, String format) {
        Date date = null;
        if (!PerStringUtils.isNullOrEmpty(str)) {
            try {
                date = new SimpleDateFormat(format).parse(str);
            } catch (ParseException e) {
                date = stringToDate(str);
            }
        }
        return date;
    }


    public static Date stringToDate(String dateStr) {
        try {
            return stringToDate(dateStr, getDefaultDateFormats());
        } catch (ParseException ignored) {
        }
        return null;
    }


    public static Date stringToDate(String dateStr, Collection<String> dateFormats) throws ParseException {
        if (dateStr == null || dateStr.length() == 0) {
            return null;
        }

        if (dateFormats == null) {
            dateFormats = getDefaultDateFormats();
        }
        Iterator<String> formatIterator = dateFormats.iterator();
        SimpleDateFormat dateParser = null;
        String format;
        while (formatIterator.hasNext()) {
            format = formatIterator.next();
            if (dateParser == null) {
                dateParser = new SimpleDateFormat(format);
            } else {
                dateParser.applyPattern(format);
            }
            try {
                return dateParser.parse(dateStr);
            } catch (ParseException pe) {

            }
        }
        throw new ParseException("Unable to parse the date " + dateStr, 0);
    }


    private static Collection<String> getDefaultDateFormats() {
        Collection<String> defaultDateFormats = new ArrayList<String>();
        defaultDateFormats.add("yyyy-MM-dd HH:mm:ss");
        defaultDateFormats.add("yyyy-MM-dd HH:mm");
        defaultDateFormats.add("yyyyMMddHHmmss");
        defaultDateFormats.add("yyyy-MM-dd");
        defaultDateFormats.add("yyyyMMddHHmm");
        defaultDateFormats.add("yyyy-MM");
        defaultDateFormats.add("yyyyMMdd");
        defaultDateFormats.add("yyyyMM");
        defaultDateFormats.add("yyyy年MM月dd日 HH时mm分ss秒");
        defaultDateFormats.add("yyyy年MM月dd日");
        defaultDateFormats.add("yyyy年MM月");
        defaultDateFormats.add("yyyy年");
        defaultDateFormats.add("yyyy");
        return defaultDateFormats;
    }


    public static Long getTimestamp(String time) {
        Long timestamp = null;
        try {
            timestamp = new SimpleDateFormat("yyyyMMddHHmmssSSS").parse(time).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return timestamp;
    }

    public static String dateToShortStringNoSep(Date date) {
        return dateToStringWithFormat(date, "yyyyMMdd");
    }


    public static String dateToLongStringNoSep(Date date) {
        return dateToStringWithFormat(date, "yyyyMMddHHmmss");
    }


    public static String dateToStringWithFormat(Date date, String format) {
        String str = "";
        if (date == null || format == null || format.length() == 0) {
            return str;
        }
        try {
            str = new SimpleDateFormat(format).format(date);
        } catch (Exception ignored) {
        }
        return str;
    }


    public static String localDateTimeToStringWithFormat(LocalDateTime dateTime, String format) {
        String str = "";
        if (dateTime == null || format == null || format.length() == 0) {
            return str;
        }
        try {
            str = DateTimeFormatter.ofPattern(format).format(dateTime);
        } catch (Exception ignored) {
        }
        return str;
    }


    /**
     * 将一种格式的日期字符转换成目标格式的字符串
     *
     * @param str          日期串
     * @param sourceFormat 原格式，如"yyyyMMddHHmmss"
     * @param destFormat   目标格式，如"yyyy-MM-dd HH:mm:ss"
     * @return String 格式化后的日期串
     * @author yangdong
     * @date 2015-8-10 下午6:09:41
     */
    public static String stringDateFormat(String str, String sourceFormat, String destFormat) {
        if (PerStringUtils.isNullOrEmpty(str)) {
            return "";
        }
        try {
            SimpleDateFormat dest = new SimpleDateFormat(destFormat);
            return dest.format(stringToDateWithFormat(str, sourceFormat));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }


    /**
     * 方法名：stringToTimeStampWithFormat
     * 作者： kane
     * 创建时间：2016-9-23 下午02:11:28
     *
     * @param dateStr    时间字符串
     * @param timeFormat 时间字符串格式，如yyyyMMddHHmmss
     * @return long型时间戳
     * 描述：将给定格式的字符串转换成时间戳
     */
    public static long stringToTimeStampWithFormat(String dateStr, String timeFormat) {
        try {
            if (dateStr != null && dateStr.length() > 0) {
                SimpleDateFormat format = new SimpleDateFormat(timeFormat);
                Date date = format.parse(dateStr);
                return date.getTime();
            }
        } catch (Exception e) {
            System.out.println("日期转换出现异常");
        }
        return 0;
    }


}
