package com.sunwin.metro.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.*;
import org.apache.hadoop.io.IOUtils;

import java.io.IOException;

/**
 * @program: sunwin_metro
 * @description: hadoop 工具类
 * @author: xuYJ
 * @create: 2021-06-17 09:48
 **/
public class FileUtil {
    private static final Log logger = LogFactory.getLog(FileUtil.class);


    public static void mergeFile(Path srcDir, final String srcFileNameExpression, String outFile, boolean overWrite, Configuration conf) throws IOException {
        FileSystem fs = null;
        FileStatus[] fileStatuses = new FileStatus[0];
        FSDataOutputStream outFileOutputStream = null;
        FSDataInputStream inputStream = null;
        try {
            fs = FileSystem.get(conf);
            fileStatuses = fs.listStatus(srcDir, new PathFilter() {
                @Override
                public boolean accept(Path path) {
                    return path.getName().contains(srcFileNameExpression);
                }
            });
            outFileOutputStream = fs.create(new Path(outFile), overWrite);
            for (FileStatus st : fileStatuses) {
                Path temp = st.getPath();
                inputStream = fs.open(temp);
                IOUtils.copyBytes(inputStream, outFileOutputStream, 4096, false);
                inputStream.close();
                logger.info("merge : " + temp.toString() + " -> " + outFile);
            }

        } catch (IOException e) {
            logger.error(e);
        } finally {
            if (fs != null) {
                fs.close();
            }
            if (fileStatuses != null) {
                fileStatuses = null;
            }
            if (outFileOutputStream != null) {
                outFileOutputStream.close();
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }

    }

    public static void main(String[] args) throws IOException {
        if (args.length < 3) {
            System.out.println("参数错误");
        }
        for (String s : args) {
            System.out.println(s);
        }
        Configuration conf = new Configuration();
        FileUtil.mergeFile(new Path(args[0]), args[1], args[2], true, conf);
    }
}

