package com.sunwin.metro.utils;

import org.ahocorasick.trie.Emit;
import org.ahocorasick.trie.Trie;

import java.io.*;
import java.util.*;

/**
 * 文件操作工具
 *
 * @author kane
 * @date 2020-02-13 13:42
 */
public class FileUtils {


    public static boolean containsWords(String inputString, String[] words) {
        boolean found = false;
        for (String word : words) {
            if (inputString.contains(word)) {
                found = true;
                break;
            }
        }
        return found;
    }


    public static <T extends Closeable> boolean close(T stream) {
        if (stream != null) {
            try {
                stream.close();
                return true;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }


    public static boolean isExistFile(String pathFileName) {
        if (!PerStringUtils.isNullOrEmpty(pathFileName)) {
            File file = new File(pathFileName);
            return file.exists() && file.isFile();
        }
        return false;
    }

    public static boolean createFile(String fileName) {
        if (PerStringUtils.isNullOrEmpty(fileName)) {
            return false;
        } else {
            File file = new File(fileName);
            try {
                return file.createNewFile();
            } catch (IOException e) {
                return false;
            }
        }
    }

    public static File getFile(String fileName) {
        if (!isExistFile(fileName)) {
            createFile(fileName);
        }
        return new File(fileName);
    }


    public static String getFileContent(String fileName, String encoding) {
        if (PerStringUtils.isNullOrEmpty(fileName) || PerStringUtils.isNullOrEmpty(encoding)) {
            return "";
        } else {
            Object[] object = getFileBufferedReader(fileName, encoding);
            if (object == null) {
                return null;
            }
            try {
                String line;
                StringBuilder sb = new StringBuilder();
                while (null != (line = ((BufferedReader) object[0]).readLine())) {
                    sb.append(line);
                }
                return sb.toString();
            } catch (Exception ignored) {
            } finally {
                close((Closeable) object[0]);
                close((Closeable) object[1]);
                close((Closeable) object[2]);
            }
            return "";
        }
    }

    private static Object[] getFileBufferedReader(String fileName, String encoding) {
        FileInputStream fileInputStream;
        InputStreamReader inputStreamReader;
        BufferedReader bufferedReader;
        if (PerStringUtils.isNullOrEmpty(fileName) || PerStringUtils.isNullOrEmpty(encoding)) {
            return null;
        } else {
            try {
                fileInputStream = new FileInputStream(getFile(fileName));
                inputStreamReader = new InputStreamReader(fileInputStream, encoding);
                bufferedReader = new BufferedReader(inputStreamReader);
            } catch (FileNotFoundException | UnsupportedEncodingException e) {
                e.printStackTrace();
                return null;
            }
        }
        return new Object[]{bufferedReader, inputStreamReader, fileInputStream};
    }


}
