package com.sunwin.metro.utils;

import com.alibaba.fastjson.JSONObject;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.HColumnDescriptor;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.*;
import org.apache.hadoop.hbase.util.Bytes;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @program: sunwin_metro
 * @description: Hbase工具类
 * @author: xuYJ
 * @create: 2021-02-20 14:31
 **/
public class HBaseUtil {

    static Connection connection = null;
    static Admin admin = null;

    static {
        Configuration conf = HBaseConfiguration.create();
        conf.set("hbase.zookeeper.quorum", "192.168.20.65:2181,192.168.20.66:2181,192.168.20.67:2181");
        try {
            connection = ConnectionFactory.createConnection(conf);
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            admin = connection.getAdmin();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void sendMessage(String tableName,JSONObject json,String family) {
        String compName = json.getString("compname");
        String stationId = json.getString("stationId");
        String station;
        String sysName;
        if (compName.contains("ACS")) {
            sysName = "001";
        } else if (compName.contains("BAS")) {
            sysName = "002";
        } else if (compName.contains("FAS")) {
            sysName = "003";
        } else if (compName.contains("PSD")) {
            sysName = "004";
        } else if (compName.contains("UPS")) {
            sysName = "005";
        } else if (compName.contains("AFC")) {
            sysName = "006";
        } else {
            sysName = "000";
        }
        if (stationId.contains("sw:404")) {
            station = "001";
        } else {
            station = "000";
        }
        String deviceTime = DateUtils.stringDateFormat(json.getString("stime"), "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'", "yyyyMMddHHmmssSSS").substring(0, 8);
        String dpe = json.getString("dpes");
        String device = String.valueOf(Math.abs(dpe.hashCode()));
        String rowKey = station + sysName + deviceTime + device;
        HashMap<String, Object> map = new HashMap<>();
        map.put("collectTime", json.getString("collectTime"));
        map.put("compDes", json.getString("compDes"));
        map.put("dpe", dpe);
        map.put("subSys", json.getString("subSys"));
        map.put("stationName", json.getString("stationName"));
        map.put("sTime", json.getString("stime"));
        map.put("compName", json.getString("compname"));
        map.put("stationId", json.getString("stationId"));
        putMapDataByRowKey(tableName, family, map, rowKey);
    }

    public static boolean tableExist(String tableName) throws IOException {
        return admin.tableExists(TableName.valueOf(tableName));
    }

    private static void createTable(String tableName, List<String> columnFamily) throws IOException {
        if (!tableExist(tableName)) {
            HTableDescriptor hTableDescriptor = new HTableDescriptor(TableName.valueOf(tableName));
            for (String f : columnFamily) {
                HColumnDescriptor hColumnDescriptor = new HColumnDescriptor(f);
                hTableDescriptor.addFamily(hColumnDescriptor);
            }
            admin.createTable(hTableDescriptor);
        }
    }

    public static Table getTable(String tableName,List<String> familyList) {
        Table tblName = null;
        try {
            createTable(tableName,familyList);
            tblName = connection.getTable(TableName.valueOf(tableName));
        } catch (IOException e) {
            e.printStackTrace();
        }
        return tblName;
    }

    public static void putDataByRowKey(String tableName, String family, String colName, String colValue, String rowKey,List<String> familyList) {
        Table table = getTable(tableName,familyList);

        try {
            Put put = new Put(rowKey.getBytes());
            put.addColumn(family.getBytes(), Bytes.toBytes(colName), colValue.getBytes());
            table.put(put);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                table.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void putMapDataByRowKey(String tableName, String family, Map<String, Object> map, String rowKey) {

        Table table = getTable(tableName, Collections.singletonList(family));
        try {
            Put put = new Put(rowKey.getBytes());
            for (String key : map.keySet()) {

                put.addColumn(family.getBytes(), key.getBytes(), map.get(key).toString().getBytes());
            }
            table.put(put);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                table.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


    public static void close(Connection connection, Admin admin) throws IOException {
        if (connection != null) {
            connection.close();
        }
        if (admin != null) {
            admin.close();
        }
    }
}
