package com.sunwin.metro.utils;


import com.alibaba.fastjson.JSONObject;
import org.apache.kafka.clients.producer.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Map;
import java.util.Properties;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Future;

/**
 * @author xuYJ
 * @description:kafka工具类
 * @date 2020/8/17 14:45
 */
public class KafkaUtils implements Serializable {

    private static final Map<String, KafkaProducer<String, String>> PRODUCER_CACHE = new ConcurrentHashMap<>();

    /**
     * 创建生产者
     */
    private static KafkaProducer<String, String> createProducer(String brokers) {
        Properties prop = new Properties();
        prop.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, brokers);
        //ACK机制
        prop.put(ProducerConfig.ACKS_CONFIG, "-1");
        prop.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        prop.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        return new KafkaProducer<>(prop);
    }

    static {
        Runtime.getRuntime().addShutdownHook(
                new Thread(() -> PRODUCER_CACHE.forEach(KafkaUtils::accept)
                )
        );
    }

    private static KafkaProducer<String, String> getProducer(String brokers) {
        return PRODUCER_CACHE.compute(brokers, (k, oldProducer) -> {
            if (oldProducer == null) {
                oldProducer = createProducer(brokers);
            }
            return oldProducer;
        });
    }


    public static Future<RecordMetadata> send(String brokers, String topic, String key, String message) {
        KafkaProducer<String, String> producer = getProducer(brokers);
        ProducerRecord<String, String> producerRecord = new ProducerRecord<>(topic, key, message);
        return producer.send(producerRecord);
    }


    public static void send(String brokers, String topic, String message) {
        KafkaProducer<String, String> producer = getProducer(brokers);
        ProducerRecord<String, String> producerRecord = new ProducerRecord<String, String>(topic, message);
        producer.send(producerRecord, new Callback() {
            @Override
            public void onCompletion(RecordMetadata recordMetadata, Exception exception) {
                if (null != exception) {
                    System.out.println("Send message exception:" + exception);
                }
                if (null != recordMetadata) {
                    System.out.println(String.format("偏移量:%s,分区:%s", recordMetadata.offset(), recordMetadata.partition()));
                }
            }
        });
    }



    private static void accept(String key, KafkaProducer<String, String> v) {
        try {
            v.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}

