package com.sunwin.metro.utils.websocket;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mysql.cj.util.StringUtils;
//import com.mysql.jdbc.StringUtils;
import com.sunwin.metro.source.mysql.MysqlData;
import com.sunwin.metro.utils.DateUtils;
import com.sunwin.metro.utils.HBaseUtil;
import com.sunwin.metro.utils.KafkaUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.java_websocket.WebSocket;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;

import java.util.Date;
import java.util.List;

/**
 * @program: sunwin_metro
 * @description: 自动重启客户端
 * @author: xuYJ
 * @create: 2021-02-18 15:57
 **/
public class Client {
    public static final Logger logger = LoggerFactory.getLogger(Client.class);
    static final String BOOTSTRAP_SERVERS = "192.168.20.65:9092,192.168.20.66:9092,192.168.20.67:9092";
    private static WebSocketClient client;

    public static void createConnect() throws Exception {
        client = new WebSocketClient(new URI("ws://127.0.0.1:12108/dpsubSys_websocket")) {
            @Override
            public void onOpen(ServerHandshake serverHandshake) {
                logger.info("webSocket服务端开始启动");
            }

            @Override
            public void onMessage(String message) {
                if (!StringUtils.isNullOrEmpty(message) && !message.contains("errorDes")) {
                    message = StringEscapeUtils.unescapeJava(message).replace("\"{", "{").replace("}\n\"", "}");
                    JSONObject object = JSON.parseObject(message);
                    object.put("collectTime", DateUtils.dateToLongStringNoSep(new Date()));
                    KafkaUtils.send(BOOTSTRAP_SERVERS, "metro_test", object.toJSONString());
                    HBaseUtil.sendMessage("metro2", object, "info");
                } else {
                    System.out.println(message);
                }
            }

            @Override
            public void onError(Exception arg0) {
                arg0.printStackTrace();
            }

            @Override
            public void onClose(int arg0, String arg1, boolean arg2) {
                try {
                    Thread.sleep(40000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

                try {
                    createConnect();
                    List<String> results = null;
                    try {
                        results = MysqlData.getWords("metro_device_all");
                    } catch (Exception exception) {
                        exception.printStackTrace();
                    }
                    JSONArray array = new JSONArray();
                    for (String result : results) {
                        array.add("sw404:" + result.trim());
                    }
                    JSONObject object = new JSONObject();
                    object.put("dpes", array);
                    client.send(object.toJSONString());

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        client.connect();
        client.setConnectionLostTimeout(600000);
        Thread.sleep(20000);
        while (!client.getReadyState().equals(WebSocket.READYSTATE.OPEN)) {
            Thread.sleep(10000);
        }
    }

    public static void send(String message) throws Exception {
        if (null == client || (!client.isOpen())) {
            createConnect();
        }
        client.send(message);
    }

}
